package ch.zhaw.pm2.racetrack.strategy;

import ch.zhaw.pm2.racetrack.Game;
import ch.zhaw.pm2.racetrack.InvalidTrackFormatException;
import ch.zhaw.pm2.racetrack.PositionVector;
import ch.zhaw.pm2.racetrack.Track;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class PathFinderStrategy implements MoveStrategy{
    private ArrayList<tryOutPaths> possiblePaths;
    private Game game;
    private boolean foundPath = false;
    private List<PositionVector.Direction> directions = Arrays.asList(PositionVector.Direction.values());
    private ArrayList<PositionVector.Direction> directionToTake = new ArrayList<>();
    private int currentMove = 0;

    public PathFinderStrategy(Game game, int carIndex) {
        this.game = game;
        possiblePaths = new ArrayList<>();
        calculatePaths();
        try {
            getDirections();
        } catch (InvalidTrackFormatException e) {
            e.printStackTrace();
        }
    }

    private void calculatePaths() {
        for (int i = 0; i < 50; i++) {
            if(foundPath){
                break;
            }

            ArrayList<tryOutPaths> temporary = new ArrayList<>();
            Iterator<tryOutPaths> it = possiblePaths.iterator();

            while (it.hasNext()) {
                tryOutPaths current = it.next();
                if (!current.isFeasible()) {
                    it.remove();
                } else {
                    for (PositionVector.Direction direction : directions) {
                        temporary.add(current);
                        if (temporary.get(temporary.size() - 1).takeDirection(direction)) {
                            foundPath = true;
                        }
                    }
                }
            }
            possiblePaths.clear();
            possiblePaths.addAll(temporary);
        }
    }

    private void getDirections() throws InvalidTrackFormatException {
        if(foundPath){
            for (tryOutPaths path: possiblePaths) {
                if (path.isFinished()){
                    directionToTake = path.getDirectionsTaken();
                }
            }
        } else {
          throw new InvalidTrackFormatException();
        }
    }

    @Override
    public PositionVector.Direction nextMove() {
        PositionVector.Direction direction = directionToTake.get(currentMove);
        currentMove++;
        return direction;
    }

    public class tryOutPaths {
        ArrayList<PositionVector.Direction>  directionsTaken = new ArrayList<>();
        PositionVector currentPosition;
        Track track;

        private boolean feasible;
        private boolean finished;
        public tryOutPaths(Track track){
            this.track = track;
        }

        public boolean isFeasible(){
            return feasible;
        }

        public boolean takeDirection(PositionVector.Direction direction) {
            if(directionsTaken.size() >= 50){
                feasible = false;
                return false;
            }
            else if(finished){
                return true;
            }
            else {
                //check if possible eventuell hier??
                directionsTaken.add(direction);
                return false;
            }
        }

        public ArrayList<PositionVector.Direction> getDirectionsTaken(){
            return directionsTaken;
        }

        public boolean isFinished(){
            return finished;
        }
    }
}
